;; ============ NumInc.lsp ===============
;;
;;  FUNCTION: Number Increment
;;  Will sequentially place numerical
;;  text upon mouse click, with optional
;;  prefix and suffix.
;;
;;  >>  Enter          =  Exit
;;  >>  Space          =  Rotate ACW
;;  >>  Shift + Space  =  Rotate CW
;;  >>  Tab            =  Rotate 90
;;  >>  Shift + Tab    =  Mirror Rotation
;;  >>  C              =  Curve Aligned
;;  >>  R              =  Replace Text/Attribute
;;
;;  << Hit Space for no Prefix/Suffix >>
;;
;;  SYNTAX: NumInc
;;
;;  AUTHOR:
;;  Copyright (c) 2009, Lee McDonnell
;;  (Contact Lee Mac, CADTutor.net)
;;
;;  PLATFORMS:
;;  Requires Express Tools,
;;  only tested in ACAD 2004.
;;
;;  VERSION:
;;  1.0  ~  12.04.2009  ~ First Release
;;  2.0  ~  14.04.2009  ~ Incremental Prefix/Suffix
;;  3.0  ~  15.04.2009  ~ DCL Version
;;  4.0  ~  15.04.2009  ~ Replace Text Added
;;  5.0  ~  16.04.2009  ~ Leading Zeros Allowed
;;
;; =======================================


(defun c:NumInc  (/ *error* vlst ovar rot pflag sflag
                    mflag pref suff snum flag CurMod
                    RepMod CurFlag RepFlag tstr tbox
                    thgt twid grdat spt btpt pt1 pt2
                    pt3 pt4 ent cObj lAng ClsPt tpPt
                    midPt dcTag layers styles OK iLay
                    istyl Strl dec inDec decpos decs)

  (vl-load-com)
  (or num:snum (setq num:snum "1"))
  (or num:ninc (setq num:ninc "1"))
  (or num:pref (setq num:pref ""))
  (or num:suff (setq num:suff ""))
  (or inc:set  (setq inc:set "sel_mid"))
  (or num:thgt (setq num:thgt (rtos (getvar "TEXTSIZE"))))

  (defun *error* (msg)
    (if ovar (mapcar 'setvar vlst ovar))
    (if (not (member msg '("Function cancelled" "quit / exit abort")))
      (princ (strcat "\nError: " msg)))
    (princ))

  (setq vlst '("DIMZIN")
        ovar (mapcar 'getvar vlst))
  (mapcar 'setvar vlst '(3))

  (setq dcTag (load_dialog "NumInc.dcl"))
  (if (not (new_dialog "Num" dcTag))
    (progn
      (alert "Error Loading Number Increment.")
      (exit)))

  (setq doc (vla-get-ActiveDocument
              (vlax-get-acad-object)))

  (vlax-for lay (vla-get-layers doc)
    (setq layers (cons (vla-get-Name lay) layers)))
  (setq layers (reverse layers))

  (vlax-for styl (vla-get-TextStyles doc)
    (setq styles (cons (vla-get-Name styl) styles)))
  (setq styles (reverse styles))

  (start_list "text_lay" 3)
  (mapcar 'add_list layers)
  (end_list)

  (start_list "text_styl" 3)
  (mapcar 'add_list styles)
  (end_list)

  (mapcar 'set_tile
          '("pref_str" "mid_str" "suff_str" "inc_str" "text_size")
          (list num:pref num:snum num:suff num:ninc num:thgt))

  (set_tile inc:set "1")

  (action_tile "sel_pref" "(setq inc:set \"sel_pref\")")
  (action_tile "sel_mid" "(setq inc:set \"sel_mid\")")
  (action_tile "sel_suff" "(setq inc:set \"sel_suff\")")
  (action_tile "accept" "(get_values) (glst) (errchk)")
  (action_tile "info" "(nest_info)")
  (action_tile "cancel" "(done_dialog)")

  (start_dialog)
  (unload_dialog dcTag)

  (if OK
    (progn
      (cond (pflag
             (setq num:pref (rtos (distof num:pref) 2 dec))
             (while (< (strlen num:pref) Strl)
               (setq num:pref (strcat (chr 48) num:pref))))
            (mflag
             (setq num:snum (rtos (distof num:snum) 2 dec))
             (while (< (strlen num:snum) Strl)
               (setq num:snum (strcat (chr 48) num:snum))))
            (sflag
             (setq num:suff (rtos (distof num:suff) 2 dec))
             (while (< (strlen num:suff) Strl)
               (setq num:suff (strcat (chr 48) num:suff)))))
      (setq rot 0.0)
      (prompt "\n<< Hit Enter to Exit >> C = Curve Aligned, Space = Rotate ACW, Tab = Rotate 90
               \nShift + Space = Rotate CW, Shift + Tab = Mirror Rotation, R = Replace")
      (while (not flag)
        (setq tstr (strcat num:pref num:snum num:suff)
              tbox (textbox (list (cons 1 tstr) (cons 7 istyl) (cons 40 (distof num:thgt))))
              thgt (- (cadadr tbox) (cadar tbox))
              twid (- (caadr tbox) (caar tbox)))
        (while (and (not CurFlag) (not RepFlag) (not flag) (/= 3 (car (setq grdat (grread t 7 0)))))
          (redraw)
          (cond
            ((and (not CurMod) (not RepMod) (= 5 (car grdat))
                  (listp (setq spt (trans (cadr grdat) 0 1))))
             (setq btpt (polar spt (+ (/ (* 3 pi) 2) rot) (/ thgt 2))
                   pt1  (polar btpt (+ pi rot) (/ twid 2))
                   pt2  (polar pt1 (+ (/ pi 2) rot) thgt)
                   pt3  (polar pt2 rot twid)
                   pt4  (polar pt1 rot twid))
             (grvecs (list 3 pt1 pt2 3 pt2 pt3 3 pt3 pt4 3 pt1 pt4)))
            ((and CurMod (not RepMod) (= 5 (car grdat))
                  (listp (setq spt (trans (cadr grdat) 0 1))))
             (if (and (setq ent (car (entsel "\nSelect Curve to Align << Hit Enter to Exit >>: ")))
                    (member (cdr (assoc 0 (entget ent)))
                            '("LINE" "ARC" "CIRCLE" "ELLIPSE" "LWPOLYLINE" "POLYLINE" "REGION" "SPLINE")))
                (progn
                  (setq cObj (vlax-ename->vla-object ent))
                  (while (/= 3 (car (setq grdat (grread t 5 0))))
                    (redraw)
                    (if (listp (setq spt (trans (cadr grdat) 0 1)))
                      (progn
                        (setq ClsPt (vlax-curve-getClosestPointto cObj spt)
                              lAng  (angle ClsPt spt)
                              btPt  (polar ClsPt lAng (/ (distof num:thgt) 2.0))
                              tpPt  (polar ClsPt lAng (+ (/ (distof num:thgt) 2.0) thgt))
                              midPt (polar btPt lAng (/ (distance btPt tpPt) 2.0))
                              pt1   (polar btPt (+ lAng (/ pi 2)) (/ tWid 2.0))
                              pt2   (polar btPt (- lAng (/ pi 2)) (/ tWid 2.0))
                              pt3   (polar tpPt (+ lAng (/ pi 2)) (/ tWid 2.0))
                              pt4   (polar tpPt (- lAng (/ pi 2)) (/ tWid 2.0)))
                        (grvecs (list 1 pt1 pt2 1 pt2 pt4 1 pt1 pt3 1 pt3 pt4)))))
                  (setq CurFlag T))
                (progn
                  (setq CurMod nil)
                  (prompt "\n<< Hit Enter to Exit >> C = Curve Aligned, Space = Rotate ACW, Tab = Rotate 90
                           \nShift + Space = Rotate CW, Shift + Tab = Mirror Rotation, R = Replace"))))
            ((and RepMod (not CurMod) (= 5 (car grdat))
                  (listp (setq spt (trans (cadr grdat) 0 1))))
             (if (and (setq ent (car (nentsel "\nSelect Text, MText or Attribute << Hit Enter to Exit >>: ")))
                      (member (cdr (assoc 0 (entget ent)))
                              '("TEXT" "MTEXT" "ATTRIB")))
               (progn
                 (entmod (subst (cons 1 tstr) (assoc 1 (entget ent)) (entget ent)))
                 (vla-regen doc acActiveViewport)
                 (setq RepFlag T))
               (progn
                 (setq RepMod nil)
                 (prompt "\n<< Hit Enter to Exit >> C = Curve Aligned, Space = Rotate ACW, Tab = Rotate 90
                          \nShift + Space = Rotate CW, Shift + Tab = Mirror Rotation, R = Replace"))))
            ((and (= 2 (car grdat)) (= 32 (cadr grdat)))
             (if (acet-sys-shift-down)
               (setq rot (- rot (/ pi 180.0)))
               (setq rot (+ (/ pi 180.0) rot))))
            ((and (= 2 (car grdat)) (= 9 (cadr grdat)))
             (if (acet-sys-shift-down)
               (setq rot (* -1.0 rot))
               (progn
                 (if (not (member rot (list 0.0 (/ pi 2) pi (/ (* 3 pi) 2))))
                   (setq rot 0.0)
                   (setq rot (+ (/ pi 2) rot))))))
            ((and (= 2 (car grdat)) (or (= 99 (cadr grdat)) (= 67 (cadr grdat))))
             (setq CurMod T))
            ((and (= 2 (car grdat)) (or (= 82 (cadr grdat)) (= 114 (cadr grdat))))
             (setq RepMod T))
            ((and (= 2 (car grdat)) (= 13 (cadr grdat))) (setq flag t))))
        (if (not flag)
          (progn
            (if (not RepMod)
              (progn
                (if CurMod
                  (progn
                    (setq lAng (- lAng (/ pi 2)))
                    (cond ((and (> lAng (/ pi 2)) (<= lAng pi))
                           (setq lAng (- lAng pi)))
                          ((and (> lAng pi) (<= lAng (/ (* 3 pi) 2)))
                           (setq lAng (+ lAng pi))))))
                (entmake (list '(0 . "TEXT")
                               (cons 8 iLay)
                               (cons 10 (if CurMod midPt spt))
                               (cons 40 (distof num:thgt))
                               (cons 1 tstr)
                               (cons 50 (if CurMod lAng rot))
                               (cons 7 istyl)
                               '(71 . 0)
                               '(72 . 1)
                               '(73 . 2)
                               (cons 11 (if CurMod midPt spt))))
                (setq CurFlag nil)))
            (setq RepFlag nil)
            (cond (pflag
                   (setq num:pref (rtos (setq pref (+ pref (distof num:ninc))) 2 dec))
                   (while (< (strlen num:pref) Strl)
                     (setq num:pref (strcat (chr 48) num:pref))))                   
                  (mflag
                   (setq num:snum (rtos (setq snum (+ snum (distof num:ninc))) 2 dec))
                   (while (< (strlen num:snum) Strl)
                     (setq num:snum (strcat (chr 48) num:snum)))) 
                  (sflag
                   (setq num:suff (rtos (setq suff (+ suff (distof num:ninc))) 2 dec))
                   (while (< (strlen num:suff) Strl)
                     (setq num:suff (strcat (chr 48) num:suff))))))))))
  (mapcar 'setvar vlst ovar)
  (princ))

(defun txt2num  (txt)
  (setq Strl (strlen txt))
  (if (setq decpos (vl-string-position 46 txt))
    (setq decs (- Strl decpos 1))
    (setq decs 0))
  (setq dec (max inDec decs))
  (cond ((and (> decs 0) (> inDec 0))
         (setq Strl (+ decpos (max decs inDec) 1)))
        ((and (zerop decs) (> inDec 0))
         (setq Strl (+ inDec 1 Strl))))
  (cond ((distof txt 5))
        ((distof txt 2))
        ((distof txt 1))
        ((distof txt 4))
        ((distof txt 3))
        (T nil)))

(defun glst  (/ x y)
  (setq x     (atoi (get_tile "text_lay"))
        iLay  (nth x layers)
        y     (atoi (get_tile "text_styl"))
        istyl (nth y styles)))

(defun get_values  (/ tmp1 tmp2 tmp3 tmp4 tmp5)
  (setq tmp1 (get_tile "pref_str")
        tmp2 (get_tile "mid_str")
        tmp3 (get_tile "suff_str")
        tmp4 (get_tile "inc_str")
        tmp5 (get_tile "text_size"))
  (or (eq tmp1 num:pref) (setq num:pref tmp1))
  (or (eq tmp2 num:snum) (setq num:snum tmp2))
  (or (eq tmp3 num:suff) (setq num:suff tmp3))
  (or (eq tmp4 num:ninc) (setq num:ninc tmp4))
  (or (eq tmp5 num:thgt) (setq num:thgt tmp5))
  (if (eq "" num:ninc)
    (setq num:ninc "0.0"))
  (if (eq "" num:thgt)
    (setq num:thgt (rtos (getvar "TEXTSIZE"))))
  (if (setq inDec (vl-string-position 46 num:ninc))
    (setq inDec (- (strlen num:ninc) inDec 1))
    (setq inDec 0)))

(defun errchk  ()
  (cond ((eq "sel_pref" inc:set)
         (if (setq pref (txt2num num:pref))
           (progn
             (setq pflag T OK T)
             (done_dialog))
           (set_tile "error" "Prefix not Numerical!")))
        ((eq "sel_mid" inc:set)
         (if (setq snum (txt2num num:snum))
           (progn
             (setq mflag T OK T)
             (done_dialog))
           (set_tile "error" "Middle not Numerical!")))
        ((eq "sel_suff" inc:set)
         (if (setq suff (txt2num num:suff))
           (progn
             (setq sflag T OK T)
             (done_dialog))
           (set_tile "error" "Suffix not Numerical!")))))

(defun nest_info  (/ dcTagi)
  (setq dcTagi (load_dialog "NumInc.dcl"))
  (if (not (new_dialog "Info" dcTagi))
    (progn
      (alert "Error Loading Information.")
      (exit)))
  (action_tile "accept" "(done_dialog)")
  (start_dialog)
  (unload_dialog dcTagi))

(princ (strcat "\nNumber Increment Loaded, type \"NumInc\" to invoke...
                \nCopyright " (chr 169) " April 2009, Lee McDonnell. "))
(princ)